import { openDB } from 'idb';
import { PrismaClient } from '@prisma/client';

const prisma = new PrismaClient();

// Tipos
export interface Transaction {
  id: string;
  amount: number;
  type: 'INCOME' | 'EXPENSE';
  category: string;
  title: string;
  date: Date;
  isRecurring: boolean;
  recurringPeriod?: 'weekly' | 'monthly' | 'yearly';
  createdAt: Date;
  updatedAt: Date;
  syncStatus: 'synced' | 'pending' | 'error';
}

// Configuración de IndexedDB
const DB_NAME = 'expenses-db';
const DB_VERSION = 1;
const STORE_NAME = 'transactions';

// Inicialización de IndexedDB
export async function initDB() {
  const db = await openDB(DB_NAME, DB_VERSION, {
    upgrade(db) {
      if (!db.objectStoreNames.contains(STORE_NAME)) {
        db.createObjectStore(STORE_NAME, { keyPath: 'id' });
      }
    },
  });
  return db;
}

// Funciones de sincronización
export async function syncTransactions() {
  const db = await openDB(DB_NAME, DB_VERSION);
  const transactions = await db.getAll(STORE_NAME);
  
  // Sincronizar con MySQL
  for (const transaction of transactions) {
    if (transaction.syncStatus === 'pending') {
      try {
        // Intentar sincronizar con MySQL
        await prisma.transaction.upsert({
          where: { id: transaction.id },
          update: {
            amount: transaction.amount,
            type: transaction.type,
            category: transaction.category,
            title: transaction.title,
            date: transaction.date,
            isRecurring: transaction.isRecurring,
            recurringPeriod: transaction.recurringPeriod,
            updatedAt: new Date(),
          },
          create: {
            id: transaction.id,
            amount: transaction.amount,
            type: transaction.type,
            category: transaction.category,
            title: transaction.title,
            date: transaction.date,
            isRecurring: transaction.isRecurring,
            recurringPeriod: transaction.recurringPeriod,
            createdAt: transaction.createdAt,
            updatedAt: transaction.updatedAt,
            syncStatus: 'synced',
          },
        });
        
        // Actualizar estado en IndexedDB
        await updateTransaction(transaction.id, { syncStatus: 'synced' });
      } catch (error) {
        console.error('Error syncing transaction:', error);
        await updateTransaction(transaction.id, { syncStatus: 'error' });
      }
    }
  }
}

// Funciones CRUD
export async function addTransaction(transaction: Omit<Transaction, 'id' | 'createdAt' | 'updatedAt' | 'syncStatus'>) {
  const db = await initDB();
  const newTransaction: Transaction = {
    ...transaction,
    id: crypto.randomUUID(),
    createdAt: new Date(),
    updatedAt: new Date(),
    syncStatus: 'pending',
  };
  
  // Guardar en IndexedDB
  await db.add(STORE_NAME, newTransaction);
  
  // Intentar sincronizar con MySQL
  try {
    await prisma.transaction.create({
      data: {
        id: newTransaction.id,
        amount: newTransaction.amount,
        type: newTransaction.type,
        category: newTransaction.category,
        title: newTransaction.title,
        date: newTransaction.date,
        isRecurring: newTransaction.isRecurring,
        recurringPeriod: newTransaction.recurringPeriod,
        createdAt: newTransaction.createdAt,
        updatedAt: newTransaction.updatedAt,
        syncStatus: 'synced',
      },
    });
    await updateTransaction(newTransaction.id, { syncStatus: 'synced' });
  } catch (error) {
    console.error('Error syncing new transaction:', error);
  }
  
  return newTransaction;
}

export async function getTransactions(): Promise<Transaction[]> {
  try {
    // Intentar obtener de MySQL primero
    const mysqlTransactions = await prisma.transaction.findMany({
      orderBy: { date: 'desc' },
    });
    
    // Actualizar IndexedDB con los datos de MySQL
    const db = await openDB(DB_NAME, DB_VERSION);
    for (const transaction of mysqlTransactions) {
      await db.put(STORE_NAME, transaction);
    }
    
    return mysqlTransactions;
  } catch (error) {
    console.error('Error fetching from MySQL, falling back to IndexedDB:', error);
    const db = await openDB(DB_NAME, DB_VERSION);
    return db.getAll(STORE_NAME);
  }
}

export async function updateTransaction(id: string, transaction: Partial<Transaction>) {
  const db = await openDB(DB_NAME, DB_VERSION);
  const existing = await db.get(STORE_NAME, id);
  if (!existing) throw new Error('Transaction not found');
  
  const updated: Transaction = {
    ...existing,
    ...transaction,
    updatedAt: new Date(),
  };
  
  // Actualizar en IndexedDB
  await db.put(STORE_NAME, updated);
  
  // Intentar sincronizar con MySQL
  try {
    await prisma.transaction.update({
      where: { id },
      data: {
        amount: updated.amount,
        type: updated.type,
        category: updated.category,
        title: updated.title,
        date: updated.date,
        isRecurring: updated.isRecurring,
        recurringPeriod: updated.recurringPeriod,
        updatedAt: updated.updatedAt,
        syncStatus: updated.syncStatus,
      },
    });
  } catch (error) {
    console.error('Error syncing update to MySQL:', error);
  }
  
  return updated;
}

export async function deleteTransaction(id: string) {
  const db = await openDB(DB_NAME, DB_VERSION);
  
  // Eliminar de IndexedDB
  await db.delete(STORE_NAME, id);
  
  // Intentar eliminar de MySQL
  try {
    await prisma.transaction.delete({
      where: { id },
    });
  } catch (error) {
    console.error('Error deleting from MySQL:', error);
  }
} 